
static void MoveContent(struct RenderList *, WORD, LONG, LONG, LONG, LONG, LONG, LONG, struct Bitmap *, WORD *);

#define Draw(a,b,c,d,e) ActionTags(AC_Draw, (a), (FLOAT)(b), (FLOAT)(c), (FLOAT)(d), (FLOAT)(e));

static UBYTE lcIgnoreChildren = FALSE;

/*****************************************************************************

###METHOD###
Name:  MoveContent
Short: Shifts the graphical content of a render object to a new position.

###DESCRIPTION###
<p>The MoveContent method provides the necessary functionality to shift the
graphical content of a render object to a new location.  The graphical area
that you shift will be limited to the area available to the render object,
i.e. other render objects that obscure the display of the target will not
be affected by the shift of content.</p>

<p>In most cases, shifting the content of a render object will leave an
uncovered area that will need to be redrawn.  The routine will automatically
redraw them for you by sending Draw messages to the render object.</p>

###INPUT###
LONG/XCoord: Horizontal coordinate of the source area.
LONG/YCoord: Vertical coordinate of the source area.
LONG/Width:  Width of the source area.
LONG/Height: Height of the source area.
LONG/XDest:  Horizontal destination within the render object.
LONG/YDest:  Vertical destination within the render object.
LONG/ClipLeft:   Clip restriction for the left edge.
LONG/ClipTop:    Clip restriction for the top edge.
LONG/ClipRight:  Clip restriction for the right edge.
LONG/ClipBottom: Clip restriction for the bottom edge.
LONG/Flags:      Optional flags.  Use MCF_INCLUDE_CHILDREN if you want to move child drawables with the graphical content.  MCF_REDRAW_UNDERLYING will send redraw messages to any drawable that is underneath the target drawable at the time of the content shift.

###END###

*****************************************************************************/

static ERROR RENDER_MoveContent(struct Render *Self, struct mtMoveContent *Args)
{
   struct RenderList *list;
   LONG index, left, top, right, bottom;
   LONG clipleft, cliptop, clipright, clipbottom;
   ERROR error;

   if (!Args) return(ObjectError(Self, MT_MoveContent, ERR_Args));

   /*** If the coordinates are unchanged, do nothing ***/

   if ((Args->XCoord IS Args->XDest) AND (Args->YCoord IS Args->YDest)) return(ERR_Okay);

   #ifdef DBG_DRAW
      DPrintF("~MoveContent()","[Render:%d] %dx%d,%dx%d TO %dx%d", Self->Head.UniqueID, Args->XCoord, Args->YCoord, Args->Width, Args->Height, Args->XDest, Args->YDest);
   #endif

   if ((list = AccessRenderList(ARF_READ, NULL))) {
      index = FindDrawableIndex(list, Self->Head.UniqueID);

      left   = list[index].AbsX + Args->XCoord;
      top    = list[index].AbsY + Args->YCoord;
      right  = left + Args->Width;
      bottom = top + Args->Height;

      clipleft   = list[index].AbsX + Args->ClipLeft;
      cliptop    = list[index].AbsY + Args->ClipTop;
      clipright  = list[index].AbsX + Args->ClipRight;
      clipbottom = list[index].AbsY + Args->ClipBottom;
      if (clipleft   < list[index].Left)   clipleft   = list[index].Left;
      if (cliptop    < list[index].Top)    cliptop    = list[index].Top;
      if (clipright  > list[index].Right)  clipright  = list[index].Right;
      if (clipbottom > list[index].Bottom) clipbottom = list[index].Bottom;

      error = MoveGraphics(Self, list, index,
         left, top, right, bottom,
         list[index].Left + Args->XDest, list[index].Top + Args->YDest,
         clipleft, cliptop, clipright, clipbottom,
         Args->Flags);

      ReleaseRenderList(ARF_READ);
      #ifdef DBG_DRAW
         StepBack();
      #endif
      return(error);
   }
   else {
      #ifdef DBG_DRAW
         StepBack();
      #endif
      return(ERR_Failed);
   }
}

/*****************************************************************************
** Internal: MoveGraphics()
**
** This function is intended to be suitable for any situtation that involves
** moving graphics from one point to another.  It takes care of all clipping
** issues for you.
**
** All coordinates that are supplied to this function must be in absolute format.
**
** Self: The drawable that represents the graphics you want to shift.
** Left/Top/Right/Bottom: The absolute coordinates of the area that you want to shift.
** DLeft/DTop: The absolute coordinates of the destination region.
** ClipX: Clipping restrictions, in absolute coordinates.
**
** NOTE: The MCF_INCLUDE_CHILDREN flag should only be used by the Render class
** Move() action.  It allows all children within the drawable to be moved as
** part of the operation, rather than attempting to avoid them.
*/

static ERROR MoveGraphics(struct Render *Self, struct RenderList *list, WORD Index,
   LONG Left, LONG Top, LONG Right, LONG Bottom, LONG DLeft, LONG DTop,
   LONG ClipLeft, LONG ClipTop, LONG ClipRight, LONG ClipBottom, LONG Flags)
{
   struct Bitmap *bitmap, *video;
   struct ClipRectangle clip, rect, clipexpose, clipvideo;
   WORD currentlevel, level, exposed, i, j, index, parentindex, root;
   BYTE redrawvolatiles;
   OBJECTID parentid;

   #ifdef DBG_DRAW
      DPrintF("~MoveGraphics()","[Render:%d:%s] Dimensions: %dx%d,%dx%d (Rel)", Self->Head.UniqueID, Self->Head.Stats->Name, Self->XCoord, Self->YCoord, Self->Width, Self->Height);
      DPrintF("MoveGraphics:","%dx%d,%dx%d TO %dx%d (Abs)", Left, Top, Right-Left, Bottom-Top, DLeft, DTop);
      DPrintF("MoveGraphics:","Clip: %dx%d,%dx%d", ClipLeft, ClipTop, ClipRight, ClipBottom);
   #endif

   if (Flags & MCF_INCLUDE_CHILDREN) lcIgnoreChildren = TRUE;
   else lcIgnoreChildren = FALSE;

   /*** Find the index of this render object in the render list ***/

   index = FindDrawableIndex(list, Self->Head.UniqueID);

   /*** Cancel if we are not visible ***/

   if (CheckVisibility(list, index) IS FALSE) {
      #ifdef DBG_DRAW
         StepBack();
      #endif
      return(ERR_Okay);
   }

   if (!list[index].ObjectID) {
      DPrintF("@MoveGraphics:","Failed to find an entry for drawable %d in the render list.", Self->Head.UniqueID);
      #ifdef DBG_DRAW
         StepBack();
      #endif
      return(ERR_Search);
   }

   clip.left   = ClipLeft;
   clip.top    = ClipTop;
   clip.right  = ClipRight;
   clip.bottom = ClipBottom;

   /* Check if the clip area falls outside of our parent(s) boundaries.  If so
   ** then we must restrict the exposed dimensions.  We check only so far as
   ** the current bitmap buffer.
   */

   parentindex = index;
   parentid    = Self->ParentID;
   while (parentid) {
      while ((list[parentindex].ObjectID != parentid) AND (parentindex > 0)) {
         parentindex--;
      }

      if (list[parentindex].BitmapID != Self->BufferID) break;

      parentid    = list[parentindex].ParentID;
      rect.left   = list[parentindex].Left;
      rect.top    = list[parentindex].Top;
      rect.right  = list[parentindex].Right;
      rect.bottom = list[parentindex].Bottom;

      ClipRectangle(&clip, &rect);
   }

   #ifdef DBG_DRAW
      DPrintF("MoveGraphics:","Clip Zone: %dx%d,%dx%d (%dw,%dh)", clip.left, clip.top, clip.right, clip.bottom, clip.right-clip.left, clip.bottom-clip.top);
   #endif

   /*** Check if the clip region is out of bounds of the parent ***/

   if ((clip.right <= clip.left) OR (clip.bottom <= clip.top)) {
      ObjectError(Self, MT_MoveContent, ERR_OutOfBounds);
      #ifdef DBG_DRAW
         StepBack();
      #endif
      return(ERR_OutOfBounds);
   }

   if (AccessObject(Self->BufferID, 5000, &bitmap) != ERR_Okay) {
      ObjectError(Self, MT_MoveContent, ERR_AccessObject);
      #ifdef DBG_DRAW
         StepBack();
      #endif
      return(ERR_AccessObject);
   }

   /* Set bitmap's clipping boundary to match the render object's dimensions, then
   ** call MoveContent() to shift the graphics.
   */

   root = FindBitmapOwner(list, index);

   bitmap->XOffset = 0;
   bitmap->YOffset = 0;
   if (root IS index) {
      bitmap->ClipLeft   = 0;
      bitmap->ClipTop    = 0;
      bitmap->ClipRight  = bitmap->Width;
      bitmap->ClipBottom = bitmap->Height;
   }
   else {
      bitmap->ClipLeft   = clip.left   - list[root].AbsX;
      bitmap->ClipTop    = clip.top    - list[root].AbsY;
      bitmap->ClipRight  = clip.right  - list[root].AbsX;
      bitmap->ClipBottom = clip.bottom - list[root].AbsY;
      if (bitmap->ClipRight  > bitmap->Width)  bitmap->ClipRight  = bitmap->Width;
      if (bitmap->ClipBottom > bitmap->Height) bitmap->ClipBottom = bitmap->Height;
   }

   exposed = FALSE;

   if ((root IS index) AND (Flags & MCF_INCLUDE_CHILDREN)); /* Do nothing to move content when shifting top-level windows */
   else MoveContent(list, index, Left, Top, DLeft, DTop, Right-Left, Bottom-Top, bitmap, &exposed);

   /* Redraw any areas that were uncovered by intersecting child drawables
   ** during the graphics shift.
   */

   if (exposed IS TRUE) {
      #ifdef DBG_DRAW
         DPrintF("~MoveGraphics:","Redraw areas uncovered by overlapping siblings and children.");
      #endif

      for (i=index+1; (list[i].ObjectID) AND (list[i].Level >= list[index].Level); i++) {
         if (list[i].BitmapID != list[index].BitmapID) goto skipcontent; /* Ignore drawables that have their own buffer space */
         if ((Flags & MCF_INCLUDE_CHILDREN) AND (list[i].Level > list[index].Level)) goto skipcontent; /* MCF_INCLUDE_CHILDREN: Means we only need to look out for intersecting siblings */
         if (!(list[i].Flags & RNF_VISIBLE)) goto skipcontent; /* Ignore non-visible drawables */
         if (list[i].Flags & RNF_REGION) goto skipcontent;
         if (list[i].Flags & RNF_INVISIBLE) continue; /* Invisibles can contain drawables, so continue rather than skipping content */

         /*** Get the absolute coordinates of the child region, relative to our buffer ***/

         rect.left   = list[i].Left;
         rect.top    = list[i].Top;
         rect.right  = list[i].Right;
         rect.bottom = list[i].Bottom;

         /*** Check if the child drawable intersects with the source area ***/

         if ((rect.right > Left) AND (rect.bottom > Top) AND (rect.left < Right) AND (rect.top < Bottom)) {
            #ifdef DBG_DRAW
               DPrintF("MoveGraphics:","Drawing uncovered area under object #%d.", list[i].ObjectID);
            #endif

            rect.left   = rect.left   - (Left - DLeft);
            rect.top    = rect.top    - (Top - DTop);
            rect.right  = rect.left   + list[i].Width;
            rect.bottom = rect.bottom + list[i].Height;
            ClipRectangle(&rect, &clip);
            RedrawDrawable(Self->Head.UniqueID, rect.left, rect.top, rect.right, rect.bottom, IRF_SINGLE_BITMAP);
         }

skipcontent:
         /*** Skip past any children of the overlapping object ***/

         for (j=i+1; list[j].Level > list[i].Level; j++);
         i = j - 1;
      }

      #ifdef DBG_DRAW
         StepBack();
      #endif
   }

   /* Determine what areas of our render object have been exposed
   ** due to obstruction from the parent/clip boundary.  Note that there
   ** are four routines, one for each direction that a rendered
   ** area can move.
   **
   ** If MCF_INCLUDE_CHILDREN is specified, then all child drawables
   ** will be redrawn as they have also been exposed.
   */

   if ((root IS index) AND (Flags & MCF_INCLUDE_CHILDREN));
   else {
#ifdef DBG_DRAW
      DPrintF("~MoveGraphics:","Redrawing areas uncovered by the clip boundary.");
#endif

      rect.Left   = Left; /* This rectangle represents the area we are copying from */
      rect.Top    = Top;
      rect.Right  = Right;
      rect.Bottom = Bottom;

      while (ExposedRegion(&rect, &clip, &clipexpose) IS TRUE) {
         clipexpose.Left   += DLeft - Left;
         clipexpose.Top    += DTop - Top;
         clipexpose.Right  += DLeft - Left;
         clipexpose.Bottom += DTop - Top;

         ClipRectangle(&clipexpose, &clip);

         if (Flags & MCF_INCLUDE_CHILDREN) {
            RedrawDrawable(Self->Head.UniqueID, clipexpose.Left, clipexpose.Top, clipexpose.Right, clipexpose.Bottom, NULL);
         }
         else RedrawDrawable(Self->Head.UniqueID, clipexpose.Left, clipexpose.Top, clipexpose.Right, clipexpose.Bottom, IRF_IGNORE_NVCHILDREN|IRF_SINGLE_BITMAP);
      }

#ifdef DBG_DRAW
      StepBack();
#endif
   }

   ReleaseObject(bitmap);

   /* Now we make our changes to the display, either by copying the
   ** buffer to the display, or by repeating our shift routine on the display
   ** directly.
   **
   ** If the buffer is in graphics memory, we use the expose routine as this
   ** will be faster than shifting the video graphics.
   **
   ** If no acceleration is available, we use the expose method (because
   ** we don't want the CPU to read from slow video RAM).
   */

   redrawvolatiles = FALSE;
   if (!(Flags & MCF_NO_EXPOSE)) {
      if ((bitmap->DataFlags & (MEM_VIDEO|MEM_BLIT)) OR (SlowVideoBlit(Self->ScreenID) IS TRUE)) {
         LONG x, y, x2, y2;

         #ifdef DBG_DRAW
            DPrintF("MoveGraphics:","Exposing video graphics instead of shifting.");
         #endif

         x  = list[index].Left;
         y  = list[index].Top;
         x2 = list[index].Right;
         y2 = list[index].Bottom;

         if (clip.left   > x)  x  = clip.left;
         if (clip.top    > y)  y  = clip.top;
         if (clip.right  < x2) x2 = clip.right;
         if (clip.bottom < y2) y2 = clip.bottom;

         ExposeDrawable(Self->Head.UniqueID, x-list[index].AbsX, y-list[index].AbsY, x2-x, y2-y, EXF_REDRAW_VOLATILE);
      }
      else {
         /* Perform the shift again, but this time do it on the video display rather
         ** than inside the rendering buffer.
         */

         parentindex = 0;
         for (i=0; (list[i].ObjectID) AND (list[i].ObjectID != Self->Head.UniqueID); i++) {
            if (list[i].ObjectID IS Self->ParentID) {
               parentindex = i;
               break;
            }
         }

         if ((video = AccessVideo(Self->ScreenID))) {

            #ifdef DBG_DRAW
               DPrintF("MoveGraphics:","Shifting video content.");
            #endif

            clipvideo = clip;

            /* Continue in adjusting the clip region so that it matches the
            ** size of the display (the earlier clip routine only went as far
            ** as the nearest bitmap buffer).
            */

            while (parentid) {
               while ((list[parentindex].ObjectID != parentid) AND (parentindex > 0)) {
                  parentindex--;
               }

               parentid = list[parentindex].ParentID;

               ClipRectangle(&clipvideo, (struct ClipRectangle *)&list[parentindex].Left);
            }

            /*** Setup the video bitmap's clipping boundary to match the render object's dimensions ***/

            video->XOffset = 0;
            video->YOffset = 0;
            video->ClipLeft   = clipvideo.left;
            video->ClipTop    = clipvideo.top;
            video->ClipRight  = clipvideo.right;
            video->ClipBottom = clipvideo.bottom;
            if (video->ClipRight  > video->Width - video->XOffset)  video->ClipRight  = video->Width - video->XOffset;
            if (video->ClipBottom > video->Height - video->YOffset) video->ClipBottom = video->Height - video->YOffset;

            exposed = FALSE;
            MoveContent(list, index, Left, Top, DLeft, DTop, Right-Left, Bottom-Top, video, &exposed);

            ReleaseVideo();

            /* Expose any areas that were not moved due to the fact that foreign
            ** regions were sitting on top of our rendered area during the graphics shift.
            */

            if (exposed IS TRUE) {
               #ifdef DBG_DRAW
                  DPrintF("MoveGraphics:","Exposing areas obscured by intersecting regions.");
               #endif

               level = list[index].Level + 1;
               i = index + 1;
               while (list[i].ObjectID) {
                  if ((list[i].Level <= level) AND (list[i].Flags & RNF_VISIBLE)) {

                     /*** Get the absolute coordinates of the foreign render object ***/

                     rect.left   = list[i].Left;
                     rect.top    = list[i].Top;
                     rect.right  = list[i].Right;
                     rect.bottom = list[i].Bottom;

                     /*** Check for an intersection ***/

                     if ((rect.left < Right) AND (rect.top < Bottom) AND (rect.right > Left) AND (rect.bottom > Top)) {
                        if (rect.left   < Left) rect.left = Left;
                        if (rect.top    < Top)  rect.top  = Top;
                        if (rect.right  > Right)  rect.right  = Right;
                        if (rect.bottom > Bottom) rect.bottom = Bottom;
                        ExposeDrawable(Self->Head.UniqueID, DLeft + (rect.left - Left) - list[index].AbsX, DTop + (rect.top - Top) - list[index].AbsY,
                           rect.right - rect.left, rect.bottom - rect.top, EXF_CHILDREN);
                     }

                     /*** Skip children of the foreign render object ***/

                     currentlevel = list[i].Level;
                     while (list[i+1].Level > currentlevel) i++;
                  }
                  i++;
               }
            }

            /* Determine what areas of our render object have been exposed
            ** due to obstruction from our parent boundary/clip area.
            */

            #ifdef DBG_DRAW
               DPrintF("~MoveGraphics:","Exposing areas obscured by the clip boundary.");
            #endif

            rect.Left   = Left;
            rect.Top    = Top;
            rect.Right  = Right;
            rect.Bottom = Bottom;

            while (ExposedRegion(&rect, &clipvideo, &clipexpose) IS TRUE) {
               clipexpose.Left   += DLeft - Left;
               clipexpose.Top    += DTop - Top;
               clipexpose.Right  += DLeft - Left;
               clipexpose.Bottom += DTop - Top;

               if (clipexpose.Left   < clipvideo.Left)   clipexpose.Left   = clipvideo.Left;
               if (clipexpose.Top    < clipvideo.Top)    clipexpose.Top    = clipvideo.Top;
               if (clipexpose.Bottom > clipvideo.Bottom) clipexpose.Bottom = clipvideo.Bottom;
               if (clipexpose.Right  > clipvideo.Right)  clipexpose.Right  = clipvideo.Right;

               ExposeDrawable(Self->Head.UniqueID, clipexpose.Left, clipexpose.Top, clipexpose.Right, clipexpose.Bottom, EXF_CHILDREN|EXF_ABSOLUTE);
            }

            #ifdef DBG_DRAW
               StepBack();
            #endif
         }

         redrawvolatiles = TRUE;
      }
   }
   else redrawvolatiles = TRUE;

   if (redrawvolatiles IS TRUE) {
      /* Any volatile drawables that are in front of our render object will need
      ** to be redrawn because their background has now been altered from the exposure.
      */

      clipexpose.left   = DLeft;
      clipexpose.top    = DTop;
      clipexpose.right  = DLeft + (Right - Left);
      clipexpose.bottom = DTop + (Bottom - Top);
      ClipRectangle(&clipexpose, &clip);

      #ifdef DBG_DRAW
         DPrintF("~MoveGraphics:","Redrawing overlapping volatiles @ %dx%d, %dx%d.", clipexpose.left, clipexpose.top, clipexpose.right, clipexpose.bottom);
      #endif

      level = list[index].Level + 1;
      for (i=index+1; (list[i].ObjectID) AND (list[i].Level > 1); i++) {
         if (list[i].Level < level) level = list[i].Level;

         if (list[i].Level <= level) {
            if ((list[i].Flags & (RNF_PRECOPY|RNF_AFTERCOPY)) AND (list[i].Flags & RNF_VISIBLE)) {
               if ((clipexpose.right > list[i].Left) AND (clipexpose.bottom > list[i].Top) AND
                   (clipexpose.left < list[i].Right) AND (clipexpose.top < list[i].Bottom)) {

                  if (!(list[i].Flags & RNF_POSTCOMPOSITE)) RedrawDrawable(list[i].ObjectID, clipexpose.left, clipexpose.top, clipexpose.right, clipexpose.bottom, NULL);

                  if (!(Flags & MCF_NO_EXPOSE)) {
                     ExposeDrawable(list[i].ObjectID, clipexpose.left, clipexpose.top, clipexpose.right, clipexpose.bottom, EXF_ABSOLUTE);
                  }
               }
            }
         }
      }

      #ifdef DBG_DRAW
         StepBack();
      #endif
   }

   /* If MCF_REDRAW_UNDERLYING is used, any area that was uncovered by the
   ** shift will be sent a redraw signal.  This is typically used when scrolling
   ** text in order to redraw the border edges.
   */

   if (Flags & MCF_REDRAW_UNDERLYING) {
      struct ClipRectangle dest;

      /*** Redraw the left/right hand strip of our parent ***/

      #ifdef DBG_DRAW
         DPrintF("~MoveGraphics()","Redrawing underlying areas that were exposed.");
      #endif

      rect.left   = Left; /* Source clip area */
      rect.top    = Top;
      rect.right  = Left + Self->Width;
      rect.bottom = Top + Self->Height;
      ClipRectangle(&rect, &clip);

      dest.left   = DLeft; /* Dest clip area */
      dest.top    = DTop;
      dest.right  = DLeft + (Right - Left);
      dest.bottom = DTop + (Bottom - Top);
      ClipRectangle(&dest, &clip);

      if (Flags & MCF_NO_EXPOSE) {
         if (root != index) RedrawNonIntersect(Self->ParentID, &dest, &rect, IRF_SINGLE_BITMAP, -1);
      }
      else {
         WORD vindex;

         for (vindex=index+1; list[vindex].Level > list[index].Level; vindex++);
         glVolatileIndex = vindex;

         if (root IS index) RedrawNonIntersect(Self->ParentID, &dest, &rect, -1, EXF_REDRAW_VOLATILE|EXF_CHILDREN);
         else RedrawNonIntersect(Self->ParentID, &dest, &rect, IRF_SINGLE_BITMAP, EXF_REDRAW_VOLATILE);

         glVolatileIndex = 0;
     }

      #ifdef DBG_DRAW
         StepBack();
      #endif
   }

   #ifdef DBG_DRAW
      StepBack();
   #endif
   return(ERR_Okay);
}

/*****************************************************************************
** Internal: MoveContent()
**
** All coordinates must be given in absolute format.
*/

static void MoveContent(struct RenderList *list, WORD Index,
                        LONG SrcLeft, LONG SrcTop, LONG DestLeft, LONG DestTop,
                        LONG Width, LONG Height, struct Bitmap *bitmap, WORD *exposed)
{
   struct ClipRectangle intersect;
   WORD i, j;
   LONG SrcRight, SrcBottom, DestRight, DestBottom;

   #ifdef DBG_DRAW
      DPrintF("~MoveContent()","[Bitmap:%d] %dx%d,%dx%d TO %dx%d", bitmap->Head.UniqueID, SrcLeft, SrcTop, Width, Height, DestLeft, DestTop);
   #endif

   for (i=Index+1; list[i].ObjectID; i++) {
      if (!(list[i].Flags & RNF_VISIBLE)) {
         /*** The drawable is not visible, skip it and its children ***/
         j = list[i].Level;
         while (list[i+1].Level > j) i++;
         continue;
      }

      if ((glIgnoreCursor) AND (list[i].Flags & RNF_CURSOR)) continue;

      if (lcIgnoreChildren IS TRUE) {
         /*** Respect only parents and siblings ***/
         if (list[i].Level > list[Index].Level) continue;
      }
      else {
         /*** Respect parents, siblings and immediate children ***/
         if (list[i].Level > list[Index].Level + 1) continue;
      }

      if (bitmap != glVideoBitmap) {
         /*** Software mode ***/
         if (list[i].BitmapID != list[Index].BitmapID) continue; /* Ignore drawables that have their own buffer space */
      }

      /*** Get the absolute coordinates of the child region ***/

      intersect.left   = list[i].Left;
      intersect.top    = list[i].Top;
      intersect.right  = list[i].Right;
      intersect.bottom = list[i].Bottom;

      /* Check if this rendered area intersects with the -destination- region.
      ** In this case, we are going to split up the source region so that it
      ** will blit around the drawable that is in the way of the destination area.
      */

      DestRight  = DestLeft + Width;
      DestBottom = DestTop  + Height;
      if ((intersect.left < DestRight) AND (intersect.top < DestBottom) AND
          (intersect.right > DestLeft) AND (intersect.bottom > DestTop)) {

         /* Adjust the coordinates of the child region to the points of intersection with
         ** the destination region.
         */

         if (intersect.left   < DestLeft)   intersect.left   = DestLeft;
         if (intersect.top    < DestTop)    intersect.top    = DestTop;
         if (intersect.right  > DestRight)  intersect.right  = DestRight;
         if (intersect.bottom > DestBottom) intersect.bottom = DestBottom;

         if (DestLeft > SrcLeft) {
            /* When content is shifting from left to right, use this edge order: Right, Top/Bottom, Left */

            if (intersect.right < DestRight) {
               /*** Split out the right edge ***/
               MoveContent(list, Index, SrcLeft + (intersect.right - DestLeft), SrcTop,
                                        intersect.right,  DestTop,
                                        DestRight - intersect.right, Height,
                                        bitmap, exposed);
            }

            if (intersect.top > DestTop) {
               /*** Split out the top edge ***/
               MoveContent(list, Index, SrcLeft + (intersect.left - DestLeft), SrcTop,
                                        intersect.left, DestTop,
                                        intersect.right - intersect.left, intersect.top - DestTop,
                                        bitmap, exposed);
            }

            if (intersect.bottom < DestBottom) {
               /*** Split out the bottom edge ***/
               MoveContent(list, Index, SrcLeft + (intersect.left - DestLeft), SrcTop+(intersect.bottom-DestTop),
                                        intersect.left, intersect.bottom,
                                        intersect.right - intersect.left, DestBottom - intersect.bottom,
                                        bitmap, exposed);
            }

            if (intersect.left > DestLeft) {
               /*** Split out the left edge ***/
               MoveContent(list, Index, SrcLeft, SrcTop,
                                        DestLeft, DestTop,
                                        intersect.left - DestLeft, Height,
                                        bitmap, exposed);
            }
         }
         else {
            /* When content is shifting right to left, use this edge order: Left, Top/Bottom, Right */

            if (intersect.left > DestLeft) {
               /*** Split out the left edge ***/
               MoveContent(list, Index, SrcLeft, SrcTop,
                                        DestLeft, DestTop,
                                        intersect.left - DestLeft, Height,
                                        bitmap, exposed);
            }

            if (intersect.top > DestTop) {
               /*** Split out the top edge ***/
               MoveContent(list, Index, SrcLeft + (intersect.left - DestLeft), SrcTop,
                                        intersect.left, DestTop,
                                        intersect.right - intersect.left, intersect.top - DestTop,
                                        bitmap, exposed);
            }

            if (intersect.bottom < DestBottom) {
               /*** Split out the bottom edge ***/
               MoveContent(list, Index, SrcLeft + (intersect.left - DestLeft), SrcTop+(intersect.bottom-DestTop),
                                        intersect.left, intersect.bottom,
                                        intersect.right - intersect.left, DestBottom - intersect.bottom,
                                        bitmap, exposed);
            }

            if (intersect.right < DestRight) {
               /*** Split out the right edge ***/
               MoveContent(list, Index, SrcLeft + (intersect.right - DestLeft), SrcTop,
                                        intersect.right,  DestTop,
                                        DestRight - intersect.right, Height,
                                        bitmap, exposed);
            }
         }

         #ifdef DBG_DRAW
            StepBack();
         #endif
         return;
      }

      /*** Check if this rendered area intersects with our -source- region ***/

      SrcRight  = SrcLeft + Width;
      SrcBottom = SrcTop + Height;
      if ((intersect.left < SrcRight) AND (intersect.top < SrcBottom) AND
          (intersect.right > SrcLeft) AND (intersect.bottom > SrcTop)) {

         if (intersect.left   < SrcLeft)   intersect.left   = SrcLeft;
         if (intersect.top    < SrcTop)    intersect.top    = SrcTop;
         if (intersect.right  > SrcRight)  intersect.right  = SrcRight;
         if (intersect.bottom > SrcBottom) intersect.bottom = SrcBottom;

         if (DestLeft > SrcLeft) {
            /* When content is shifting from left to right, use this edge order: Right, Top/Bottom, Left */

            if (intersect.right < SrcRight) {
               /*** Split out the right edge ***/
               MoveContent(list, Index, SrcLeft + (intersect.right - SrcLeft), SrcTop,
                                        DestLeft + (intersect.right - SrcLeft), DestTop,
                                        SrcRight - intersect.right, Height,
                                        bitmap, exposed);
            }

            if (intersect.top > SrcTop) {
               /*** Split out the top edge ***/
               MoveContent(list, Index, intersect.left, SrcTop,
                                        DestLeft + (intersect.left - SrcLeft), DestTop,
                                        intersect.right - intersect.left, intersect.top - SrcTop,
                                        bitmap, exposed);
            }

            if (intersect.bottom < SrcBottom) {
               /*** Split out the bottom edge ***/
               MoveContent(list, Index, intersect.left, intersect.bottom,
                                        DestLeft + (intersect.left - SrcLeft), DestTop + (intersect.bottom - SrcTop),
                                        intersect.right - intersect.left, SrcBottom - intersect.bottom,
                                        bitmap, exposed);
            }

            if (intersect.left > SrcLeft) {
               /*** Split out the left edge ***/
               MoveContent(list, Index, SrcLeft, SrcTop,
                                        DestLeft, DestTop,
                                        intersect.left - SrcLeft, Height,
                                        bitmap, exposed);
            }
         }
         else {
            /* When content is shifting from right to left, use this edge order: Left, Top/Bottom, Right */

            if (intersect.left > SrcLeft) {
               /*** Split out the left edge ***/
               MoveContent(list, Index, SrcLeft, SrcTop,
                                        DestLeft, DestTop,
                                        intersect.left - SrcLeft, Height,
                                        bitmap, exposed);
            }

            if (intersect.top > SrcTop) {
               /*** Split out the top edge ***/
               MoveContent(list, Index, intersect.left, SrcTop,
                                        DestLeft + (intersect.left - SrcLeft), DestTop,
                                        intersect.right - intersect.left, intersect.top - SrcTop,
                                        bitmap, exposed);
            }

            if (intersect.bottom < SrcBottom) {
               /*** Split out the bottom edge ***/
               MoveContent(list, Index, intersect.left, intersect.bottom,
                                        DestLeft + (intersect.left - SrcLeft), DestTop + (intersect.bottom - SrcTop),
                                        intersect.right - intersect.left, SrcBottom - intersect.bottom,
                                        bitmap, exposed);
            }

            if (intersect.right < SrcRight) {
               /*** Split out the right edge ***/
               MoveContent(list, Index, SrcLeft + (intersect.right - SrcLeft), SrcTop,
                                        DestLeft + (intersect.right - SrcLeft), DestTop,
                                        SrcRight - intersect.right, Height,
                                        bitmap, exposed);
            }
         }

         *exposed = TRUE;

         #ifdef DBG_DRAW
            StepBack();
         #endif

         return;
      }
   }

   /*** The region is not obscured, so perform the move ***/

   if (glVideoBitmap IS bitmap) {
      ActionTags(MT_BlitArea, bitmap, bitmap, NULL, SrcLeft, SrcTop, Width, Height, DestLeft, DestTop);
   }
   else {
      WORD root;
      root = FindBitmapOwner(list, Index);

      ActionTags(MT_BlitArea, bitmap, bitmap, NULL,
         SrcLeft - list[root].AbsX, SrcTop - list[root].AbsY,
         Width, Height,
         DestLeft - list[root].AbsX, DestTop - list[root].AbsY);
   }

   #ifdef DBG_DRAW
      StepBack();
   #endif
}

